#include <stdio.h>
#include <stdlib.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include "map.h"
#include "personnage.h"
#include "prof.h"
#include "DS.h"
#include "GAME.h"
/*!
 * \def WINDOW_WIDTH
 * \brief The window's width.
 */
#define WINDOW_WIDTH 700
/*!
 * \def WINDOW_HEIGHT
 * \brief The window's height.
 */
#define WINDOW_HEIGHT 700
/*!
 * \def SPEED
 * \brief General speed.
 */
#define SPEED 5


int main(int argc, char* args[])
{
	//Init SDL soft with timer
    if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) != 0)
    {
        printf("Cannot init SDL GAME, Error : %s\n", SDL_GetError());
        return 1;
    }

    // create a new window
    SDL_Window * window = SDL_CreateWindow("INSAGAME",SDL_WINDOWPOS_CENTERED,SDL_WINDOWPOS_CENTERED,WINDOW_WIDTH+2,WINDOW_HEIGHT+2,0);
    if (!window)
    {
        printf("Cannot create new window: %s\n", SDL_GetError());
        SDL_Quit();
        return 1;
    }

    // create a new renderer for graphic sofware
    Uint32 render_flags = SDL_RENDERER_ACCELERATED;
    SDL_Renderer* renderer = SDL_CreateRenderer(window, -1, render_flags);
    if (!renderer)
    {
        printf("Cannot create renderer: %s\n", SDL_GetError());
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }

	//Create Map and give it the functional matrix
	MAP map_game;

    init_map(&map_game,"ressource/colored.bmp","ressource/mapvis.bmp");
    map_game.functional = maptomatrix(&map_game);

	//Create a texture for map
    SDL_Texture * texture_map = IMG_LoadTexture(renderer, map_game.visual_path);
	
    if (!texture_map)
    {
        printf("error creating texture for map: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }
	
	//create a Personnage "hero" and query his texture
	Personnage hero;

	init_pers(&hero,3,480,28,23,SPEED,"ressource/man.png");
	
    // load the image data into the graphics hardware's memory
    SDL_Texture* texture_hero = IMG_LoadTexture(renderer, hero.image_pers);

    if (!texture_hero)
    {
        printf("Cannot create texture for persson: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }

    // struct to hold the position and size of the sprite
    SDL_Rect hero_rect;

    // get and scale the dimensions of texture
    SDL_QueryTexture(texture_hero, NULL, NULL, &hero_rect.w, &hero_rect.h);
    hero_rect.w =  hero.larg_pers;
    hero_rect.h =  hero.long_pers;


	//create a Prof "teacher" and query his texture
	Prof teacher;

	init_prof(&teacher,350,380,40,25,SPEED,"ressource/prof.png");
	
    // load the image data into the graphics hardware's memory
	
    SDL_Texture* texture_teacher = IMG_LoadTexture(renderer, teacher.image_prof);


    if (!texture_teacher)
    {
        printf("Cannot create texture for prof: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }

	 // struct to hold the position and size of prof
	SDL_Rect teacher_rect;

	// get and scale the dimensions of texture
    SDL_QueryTexture(texture_teacher, NULL, NULL, &teacher_rect.w, &teacher_rect.h);
    teacher_rect.w = teacher.larg_prof;
    teacher_rect.h = teacher.long_prof;
    teacher_rect.x = teacher.x_prof;
    teacher_rect.y = teacher.y_prof;


	//Create a DS "exam" and its texture
	DS exam;
	
	fflush(stdin);
	int r = 2 + rand() % 9;       //random [2,10] because bug door 1 impossible => for complete 11->30
	
	init_DS(&exam,r,"ressource/DS.png");

    // load the image data into the graphics hardware's memory
    SDL_Texture* texture_DS = IMG_LoadTexture(renderer, exam.image_DS);

    if (!texture_DS)
    {
        printf("error creating texture: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }

    SDL_Rect exam_rect;

    SDL_QueryTexture(texture_DS, NULL, NULL, &exam_rect.w, &exam_rect.h);
    exam_rect.w /= 30;
    exam_rect.h /= 35;
    exam_rect.x=exam.x_DS;
    exam_rect.y=exam.y_DS;

    //variables for inputs storage
    int up = 0;
    int down = 0;
    int left = 0;
    int right = 0;

    //collision variable
    int collision;

    //result variable
    int result = 0;

    // set to 1 when window close button is pressed
    int close_request = 0;

    // game loop
    while (!close_request)
    {
        // process events
        SDL_Event event;
        while (SDL_PollEvent(&event))
        {
            switch (event.type)
            {
                case SDL_QUIT:
                    close_request = 1;
                    break;
                case SDL_KEYDOWN:
                    switch (event.key.keysym.scancode)
                    {
                        case SDL_SCANCODE_UP:
                            up = 1;
                            break;
                        case SDL_SCANCODE_LEFT:
                            left = 1;
                            break;
                        case SDL_SCANCODE_DOWN:
                            down = 1;
                            break;
                        case SDL_SCANCODE_RIGHT:
                            right = 1;
                            break;
                    }
                    break;
                case SDL_KEYUP:
                    switch (event.key.keysym.scancode)
                    {
                        case SDL_SCANCODE_UP:
                            up = 0;
                            break;
                        case SDL_SCANCODE_LEFT:
                            left = 0;
                            break;
                        case SDL_SCANCODE_DOWN:
                            down = 0;
                            break;
                        case SDL_SCANCODE_RIGHT:
                            right = 0;
                            break;
                    }
                    break;
            }
        }

        // determine position following inputs
        if (up && !down && !right && !left) hero.y_pers -= SPEED;
        if (!up && down && !right && !left) hero.y_pers += SPEED;
        if (!up && !down && !right && left) hero.x_pers -= SPEED ;
        if (!up && !down && right && !left) hero.x_pers += SPEED;

		//recalculate 4 points of hero
		calcul_points_pers(&hero);
		
        // collision detection
        collision = test_position(&hero,&teacher,&exam ,&map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds

        // set the positions following collision variable
        switch (collision)
        {
            case 0:
                hero_rect.x = (int) hero.x_pers;
                hero_rect.y = (int) hero.y_pers;
                break;
            case 1:
                hero.y_pers = hero_rect.y;
                hero.x_pers = hero_rect.x;
                break;
            case 2:
                close_request = 1;
                result = 1;
                break;

            case 3:
                close_request = 1;
                result = 0;
                break;

            case 4:
                if (hero.x_pers <= 0) hero.x_pers = 2;
                if (hero.y_pers <= 0) hero.y_pers = 2;
                if (hero.x_pers >= WINDOW_WIDTH - hero_rect.w) hero.x_pers = WINDOW_WIDTH - hero_rect.w-2;
                if (hero.y_pers >= WINDOW_HEIGHT - hero_rect.h) hero.y_pers = WINDOW_HEIGHT - hero_rect.h-2;
                hero_rect.x = (int) hero.x_pers;
                hero_rect.y = (int) hero.y_pers;
                break;
        }

        // clear the window
        SDL_RenderClear(renderer);

        // draw the image to the window
        SDL_RenderCopy(renderer, texture_map, NULL, NULL);
        SDL_RenderCopy(renderer, texture_hero, NULL, &hero_rect);
        SDL_RenderCopy(renderer, texture_teacher, NULL, &teacher_rect);
        SDL_RenderCopy(renderer, texture_DS, NULL, &exam_rect);
        SDL_RenderPresent(renderer);

        // wait 17ms
		SDL_Delay(17);
    }

	SDL_DestroyTexture(texture_map);
	SDL_DestroyTexture(texture_hero);
	SDL_DestroyTexture(texture_teacher);
	SDL_DestroyTexture(texture_DS);
	
	//texture for endgame
	SDL_Texture* texture_endgame;
	
	if(result==1){
		texture_endgame = IMG_LoadTexture(renderer,"ressource/win.png");
	}
	else texture_endgame = IMG_LoadTexture(renderer,"ressource/lose.png");
	
    if (!texture_endgame)
    {
        printf("error creating texture: %s\n", SDL_GetError());
        SDL_DestroyRenderer(renderer);
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }
    
	SDL_RenderCopy(renderer, texture_endgame, NULL, NULL);
	SDL_RenderPresent(renderer);
	SDL_Delay(2000);

    // clean up resources before exiting
	SDL_DestroyTexture(texture_endgame);
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();

    return 0;
}
